package de.duehl.vocabulary.japanese.launcher.logic;

import de.duehl.basics.io.BatchFileWithOwnJarActualiser;
import de.duehl.basics.io.FileHelper;
import de.duehl.basics.system.SystemTools;
import de.duehl.basics.system.launcher.JarStarter;
import de.duehl.basics.system.launcher.NewestJarDeterminer;
import de.duehl.basics.text.handling.BooleanHandling;
import de.duehl.basics.version.Version;
import de.duehl.html.download.binary.SimpleBinaryFileDownloader;
import de.duehl.swing.ui.GuiTools;
import de.duehl.vocabulary.japanese.common.launcher.SecretParameterForTrainerStart;
import de.duehl.vocabulary.japanese.common.persistence.Options;
import de.duehl.vocabulary.japanese.common.persistence.SessionManager;
import de.duehl.vocabulary.japanese.common.tools.VocabularyPathHelper;
import de.duehl.vocabulary.japanese.common.website.VocaluaryTrainerWebsiteConstants;
import de.duehl.vocabulary.japanese.common.website.update.program.ProgramUpdaterTools;
import de.duehl.vocabulary.japanese.launcher.VocabularyTrainerLauncherVersion;
import de.duehl.vocabulary.japanese.launcher.logic.checks.MissingExternalJarsChecker;
import de.duehl.vocabulary.japanese.launcher.logic.install.Installer;
import de.duehl.vocabulary.japanese.launcher.logic.oldjars.OldLauncherJarsRemover;
import de.duehl.vocabulary.japanese.launcher.logic.selfcheck.LauncherSelfUpdateChecker;
import de.duehl.vocabulary.japanese.launcher.ui.LauncherGui;
import de.duehl.vocabulary.japanese.launcher.update.vocables.VocablesUpdater;

import static de.duehl.vocabulary.japanese.common.website.download.WebsiteFileContentDownloader.NO_DOWNLOAD_SUCCESS;

import java.util.List;

import javax.swing.SwingUtilities;

/**
 * Diese Klasse stellt die Logik des Launchers des Vokabeltrainers dar.
 *
 * @version 1.01     2024-12-11
 * @author Christian Dühl
 */

public class LauncherLogic {

    /** Der Anfang der Dateinamen der Jars mit dem Vokabeltrainer. */
    private final static String TRAINER_JAR_NAME_START = "de.duehl.vocabulary.japanese_";


    /**
     * Das Verzeichnis in dem der Launcher und der Vokabeltrainer zusammen mit den externen Jars
     * liegen.
     */
    private final String programDir;

    /** Die grafische Oberfläche des Launchers. */
    private LauncherGui gui;

    /** Gibt an, ob der Launcher aktuell ist, und also danach fortfahren soll. */
    private boolean launcherIsActual;

    /** Die Version des Launchers für die Batchdateien. */
    private String launcherVersion;

    /** Gibt an, ob der Launcher das erste Mal auf diesem Computer gestartet wurde. */
    private boolean firstLaunchOnThisComputer;

    /**
     * Die Versionsnummer der neusten installierten Jar des Vokabel-Trainers.
     *
     * Wenn sie nicht ermittelt werden konnte, ist sie leer.
     */
    private String newestTrainerVersionNumberInstalled;

    /**
     * Die Versionsnummer auf dem Server.
     *
     * Wenn sie nicht ermittelt werden konnte, ist sie leer des Vokabel-Trainers.
     */
    private String trainerVersionNumberOnServer;

    /** Konstruktor. */
    public LauncherLogic() {
        programDir = SystemTools.getCurrentWorkingDirectory();
        newestTrainerVersionNumberInstalled = "";
    }

    /** Führt den Launcher aus. */
    public void launch() {
        createAndShowGui();
        checkForSelfUpdate();
        if (launcherIsActual) {
            determineIfFirstLaunchOnThisComputer();
            startInstallationProcessIfFirstLaunchOnThisComputer();
            removeOldLauncherJars();
            checkForMissingExternalJars();
            perhapsDownloadNewVocables();
            /*
             * Da der Download der Vokabeln in einem eigenen Thread erfolgt, findet alles danach
             * in der Methode
             *     runAfterVocaublaryVersionCheckAndPerhapsDownloadInEdt()
             * statt.
             *
             * Da dort Dinge im eigenen Thread laufen, wird closeGui an beiden Stellen aufgerufen,
             * einmal dort, und einmal hier im else-Zweig..
             */
        }
        else {
            updateBatchFiles(); // Gerade hier, da nun ein neuer Launcher runtergeladen wurde!
            closeGui();
        }
    }

    private void createAndShowGui() {
        gui = new LauncherGui();
        gui.appendMessage("Start des Launchers (Version "
                + VocabularyTrainerLauncherVersion.getOnlyVersion() + ")");
        gui.setVisible(true);
    }

    private void checkForSelfUpdate() {
        gui.appendMessage("Suche nach neueren Versionen des Launchers online ...");

        LauncherSelfUpdateChecker checker = new LauncherSelfUpdateChecker();
        checker.check();
        launcherVersion = checker.getLauncherVersion();
        launcherIsActual = checker.isLauncherActual();
    }

    /**
     * Ermittelt, ob der Launcher das erste Mal auf dem Rechner läuft.
     *
     * Da im Sessionmanager bereits statisch das VOCABLE_TRAINER_DIRECTORY-Verzeichnis erzeugt wird,
     * gibt es das ggf. schon. In diesem Fall wird geprüft, ob das SessionFile enthält.
     */
    private void determineIfFirstLaunchOnThisComputer() {
        gui.appendMessage("Prüfe ob der Launcher das erste Mal auf dem Rechner läuft ...");

        if (FileHelper.isDirectory(SessionManager.VOCABLE_TRAINER_DIRECTORY)) {
            gui.appendMessage("    das benutzerabhängige Verzeichnis ist vorhanden");
            if (FileHelper.isFile(SessionManager.SESSION_FILENAME)) {
                gui.appendMessage("    das Session-File im benutzerabhängigen Verzeichnis ist "
                        + "vorhanden");
                firstLaunchOnThisComputer = false;
            }
            else {
                gui.appendMessage("    das Session-File im benutzerabhängigen Verzeichnis ist "
                        + "nicht vorhanden");
                firstLaunchOnThisComputer = true;
            }
        }
        else {
            gui.appendMessage("    das benutzerabhängige Verzeichnis ist nicht vorhanden");
            firstLaunchOnThisComputer = true;
        }

        gui.appendMessage("    -> " + BooleanHandling.booleanToString(firstLaunchOnThisComputer));
    }

    private void startInstallationProcessIfFirstLaunchOnThisComputer() {
        if (firstLaunchOnThisComputer) {
            gui.appendMessage("Starte Installation ...");
            startInstallationProcess();
        }
        else {
            gui.appendMessage("Der Launcher läuft nicht zum ersten Mal.");
        }
    }

    private void startInstallationProcess() {
        Installer installer = new Installer(gui);
        installer.install();
    }

    /** Prüft ob es eine neuere Version von Vokabeln gibt und lädt sie ggf. herunter. */
    private void perhapsDownloadNewVocables() {
        gui.appendMessage("Prüfe auf neuere Version der Vokabularien ...");

        Options options = VocabularyPathHelper.loadOptionsOutsideFromVocableTrainer();
        VocablesUpdater updater = new VocablesUpdater(options, gui, firstLaunchOnThisComputer,
                () -> runAfterVocaublaryVersionCheckAndPerhapsDownload());
        updater.update();
    }

    private void runAfterVocaublaryVersionCheckAndPerhapsDownload() {
        SwingUtilities.invokeLater(() -> runAfterVocaublaryVersionCheckAndPerhapsDownloadInEdt());
    }

    private void runAfterVocaublaryVersionCheckAndPerhapsDownloadInEdt() {
        determineNewestTrainerVersionNumberInstalled();
        determineTrainerVersionNumberOnServer();
        determineToPerhapsDownloadAndToLaunchTrainer();
        updateBatchFiles();
        closeGui();
    }

    private void removeOldLauncherJars() {
        gui.appendMessage("Suche alte Versionen des Launchers auf der lokalen Festplatte ...");

        OldLauncherJarsRemover remover = new OldLauncherJarsRemover();
        remover.findAndRemove();
        int numberOfRemovedJars = remover.getNumberOfRemovedJars();
        gui.appendMessage("    Anzahl entfernter Jars: " + numberOfRemovedJars);
    }

    private void checkForMissingExternalJars() {
        MissingExternalJarsChecker checker = new MissingExternalJarsChecker(gui);
        checker.checkForMissingExternalJars();
    }

    private void determineNewestTrainerVersionNumberInstalled() {
        gui.appendMessage("Ermittle die installierte Version des Vokabeltrainers ...");

        NewestJarDeterminer determiner = new NewestJarDeterminer(TRAINER_JAR_NAME_START);
        determiner.removeOldJars();
        determiner.determine();
        if (determiner.isJarFound()) {
            String newestJarFilename = determiner.getNewestJarFilename();
            newestTrainerVersionNumberInstalled = Version
                    .extractVersionFromJarFilename(newestJarFilename, TRAINER_JAR_NAME_START);
            gui.appendMessage("    -> " + newestTrainerVersionNumberInstalled);
        }
        else {
            newestTrainerVersionNumberInstalled = "";
            gui.appendMessage("    konnte nicht ermittelt werden.");
        }
    }

    private void determineTrainerVersionNumberOnServer() {
        gui.appendMessage("Ermittle die Version des Vokabeltrainers auf dem Webserver ...");

        trainerVersionNumberOnServer = ProgramUpdaterTools.downloadProgramVersionFromServer();
        if (trainerVersionNumberOnServer.equals(NO_DOWNLOAD_SUCCESS)) {
            trainerVersionNumberOnServer = "";
            gui.appendMessage("    konnte nicht ermittelt werden.");
        }
        else {
            gui.appendMessage("    -> " + trainerVersionNumberOnServer);
        }
    }

    private void determineToPerhapsDownloadAndToLaunchTrainer() {
        gui.appendMessage("Prüfe ob der Trainer ggf. herunterzuladen ist und starten dann den "
                + "passenden Trainer ...");

        boolean installedVersionAvailable = !newestTrainerVersionNumberInstalled.isEmpty();
        boolean versionOnWebsiteAvailable = !trainerVersionNumberOnServer.isEmpty();

        if (installedVersionAvailable && versionOnWebsiteAvailable) {
            gui.appendMessage("    beide Versionen sind vorhanden");

            if (isTrainerVersionOnServerNewer()) {
                if (userWantsToDownloadNewerTrainerVersion()) {
                    downloadTrainerFromServerAndStartIt();
                }
                else {
                    startNewestInstalledTrainer();
                }
            }
            else {
                startNewestInstalledTrainer();
            }
        }
        else if (installedVersionAvailable) {
            gui.appendMessage("    nur die lokale Version ist vorhanden");
            startNewestInstalledTrainer();
        }
        else if (versionOnWebsiteAvailable) {
            gui.appendMessage("    nur die Version auf dem Webserver ist vorhanden");
            if (userWantsToDownloadMissingTrainerJar()) {
                downloadTrainerFromServerAndStartIt();
            }
        }
        else {
            gui.appendMessage("    keine Version ist vorhanden");
            informUserWeHaveNoTrainerToStart();
        }
    }

    private boolean isTrainerVersionOnServerNewer() {
        Version versionOnServer = new Version(trainerVersionNumberOnServer, "interessiert nicht");
        Version versionInstalled = new Version(newestTrainerVersionNumberInstalled,
                "interessiert nicht");
        return versionOnServer.isNewerThan(versionInstalled);
    }

    private boolean userWantsToDownloadNewerTrainerVersion() {
        String title = "Es gibt eine neuere Version des Vokabeltrainers auf dem Server";
        String message =  "Es gibt eine neuere Version des Vokabeltrainers auf dem Server:\n\n"
                + "Installierte Version: " + newestTrainerVersionNumberInstalled + "\n"
                + "Version auf dem Server: " + trainerVersionNumberOnServer + "\n\n"
                + "Möchten Sie die neue Version herunterladen?";
        return GuiTools.askUser(title, message);
    }

    private void downloadTrainerFromServerAndStartIt() {
        gui.appendMessage("    Lade neue Programmversion des Vokabeltrainers vom Webserver ...");

        String jarBareFilename = createLocalTrainerBareJarFilenameFromVersion(
                trainerVersionNumberOnServer);
        String jarFileUrl = VocaluaryTrainerWebsiteConstants.BASE_URL + jarBareFilename;
        String localJarFilename = FileHelper.concatPathes(programDir, jarBareFilename);
        SimpleBinaryFileDownloader.downloadBinaryFile(jarFileUrl, localJarFilename);
        removeOldTrainerJar();

        /*
         * Da nun ein neuer Trainer heruntergeladen wurde, setze ich die Version auf die
         * heruntergeladene, damit ich diese starten kann und damit beim Aktualisieren der
         * Batchdateien nicht die alte Version eingetragen wird:
         */
        newestTrainerVersionNumberInstalled = trainerVersionNumberOnServer;
        startNewestInstalledTrainer();
    }

    private static String createLocalTrainerBareJarFilenameFromVersion(String version) {
        return TRAINER_JAR_NAME_START + version + ".jar";
    }

    private void removeOldTrainerJar() {
        if (!newestTrainerVersionNumberInstalled.isEmpty()) {
            gui.appendMessage("    Entferne alte Jar des Vokabeltrainers mit der Version "
                    + newestTrainerVersionNumberInstalled + " ...");
            String oldJarBareFilename = createLocalTrainerBareJarFilenameFromVersion(
                    newestTrainerVersionNumberInstalled);
            FileHelper.deleteFile(oldJarBareFilename);
        }
    }

    private void startNewestInstalledTrainer() {
        gui.appendMessage("    Starte den neusten installierten Vokabeltrainer ...");

        String jarBareFilename = createLocalTrainerBareJarFilenameFromVersion(
                newestTrainerVersionNumberInstalled);
        String localJarFilename = FileHelper.concatPathes(programDir, jarBareFilename);

        JarStarter starter = new JarStarter(localJarFilename,
                SecretParameterForTrainerStart.SECRET_PARAMETER);
        starter.start();
        boolean success = starter.isSuccess();
        String command = starter.getCommand();
        if (!success) {
            String title = "Der Start der Jar-Datei hatte keinen Erfolg";
            String message =  "Der Start der Jar-Datei hatte keinen Erfolg:\n"
                    + "Abgeschickter Befehl: " + command;
            GuiTools.informUser(title, message);
        }

        /*
         * Es wäre interessant zu wissen, ob die einfach mit vollem Pfad gestartete Jar dann mit
         *     SystemTools.getCurrentWorkingDirectory();
         * auch wieder das richtige currentWorkingDirectory angeben  würde und nicht
         *     C:\\
         * oder sowas.
         *
         * Aber da der eigentliche Vokabeltrainer es nicht nutzt, ist das erstmal egal.
         */
    }

    private boolean userWantsToDownloadMissingTrainerJar() {
        String title = "Vokabeltrainer heruunterladen?";
        String message =  ""
                + "Es gibt keine lokale Version des Vokabeltrainer, aber eine auf dem Server:\n\n"
                + "Installierte Version: ---\n"
                + "Version auf dem Server: " + trainerVersionNumberOnServer + "\n\n"
                + "Möchten Sie die neue Version herunterladen?";
        return GuiTools.askUser(title, message);
    }

    private void informUserWeHaveNoTrainerToStart() {
        String title = "Der Launcher kann den Vokabeltrainer nicht starten";
        String message =  "Es liegt keine installierte Jar des Vokabeltrainers vor und auch "
                + "von der Webseite\n"
                + "ist gerade keine erhältlich. Bitte probieren Sie es später noch einmal.";
        GuiTools.informUser(title, message);
    }

    private void updateBatchFiles() {
        gui.appendMessage("Aktualisiere Batchdateien ...");

        updateLauncherBatchFiles();
        if (!newestTrainerVersionNumberInstalled.isEmpty()) {
            updateTrainerBatchFiles();
        }
    }

    private void updateLauncherBatchFiles() {
        gui.appendMessage("    Aktualisiere Batchdateien mit dem Launcher ...");

        updateBatchFiles("de.duehl.vocabulary.japanese.launcher", launcherVersion);
    }

    private void updateTrainerBatchFiles() {
        gui.appendMessage("    Aktualisiere Batchdateien mit dem Vokabeltrainer ...");

        updateBatchFiles("de.duehl.vocabulary.japanese", newestTrainerVersionNumberInstalled);
    }

    private void updateBatchFiles(String name, String version) {
        BatchFileWithOwnJarActualiser actualiser = new BatchFileWithOwnJarActualiser(programDir,
                name, version);
        actualiser.actualize();
        List<String> actualisedFilenames = actualiser.getActualisedFilenames();

        List<String> actualisedBareFilenames =
                FileHelper.filenamesToBareFilenames(actualisedFilenames);
        for (String actualizedBareFilename : actualisedBareFilenames) {
            gui.appendMessage("        " + actualizedBareFilename + " wurde aktualisiert (wenn "
                    + "auch ggf. mit der gleichen Version wie zuvor).");
        }
    }

    private void closeGui() {
        System.out.println(gui.getSplashText());
        gui.closeDialog();
    }

}
