package de.duehl.vocabulary.japanese.launcher.logic.checks;

import java.util.ArrayList;
import java.util.List;

import de.duehl.basics.collections.CollectionsHelper;
import de.duehl.basics.io.FileHelper;
import de.duehl.basics.system.SystemTools;
import de.duehl.vocabulary.japanese.common.ui.dialogs.DialogWithCopyableUrl;
import de.duehl.vocabulary.japanese.launcher.ui.LauncherGui;

import static de.duehl.vocabulary.japanese.common.website.VocaluaryTrainerWebsiteConstants.SHORT_VOCBULARY_TRAINER_WEBPAGE_URL;

/**
 * Diese Klasse prüft, ob alle benötigten externen Jars vorhanden sind.
 *
 * @version 1.01     2024-12-08
 * @author Christian Dühl
 */

public class MissingExternalJarsChecker {

    /** Die grafische Oberfläche des Launchers. */
    private final LauncherGui gui;

    /**
     * Das Verzeichnis in dem der Launcher und der Vokabeltrainer zusammen mit den externen Jars
     * liegen.
     */
    private final String programDir;

    /**
     * Konstruktor.
     *
     * @param gui
     *            Die grafische Oberfläche des Launchers.
     */
    public MissingExternalJarsChecker(LauncherGui gui) {
        this.gui = gui;

        programDir = SystemTools.getCurrentWorkingDirectory();
    }

    private static final List<String> EXTERNAL_JARS = CollectionsHelper.buildListFrom(
            "commons-logging-1.2.jar", "httpclient-4.5.3.jar", "httpcore-4.4.6.jar",
            "jl1.0.1.jar", "jsoup-1.10.2.jar", "zip4j-2.11.5.jar");
            // "jsch-0.1.55.jar" brauche ich nur für SFTP für den Uploader, also brauchen ihn
            // normale Anwender nicht...

    /** Testen ob die benötigten externen Jars vorhanden sind. */
    public void checkForMissingExternalJars() {
        gui.appendMessage("    überprüfe benötigte externe Jar-Dateien ...");

        List<String> missingJars = new ArrayList<>();

        for (String extrernalJar : EXTERNAL_JARS) {
            String externalJarWithDir = FileHelper.concatPathes(programDir, extrernalJar);
            if (!FileHelper.isFile(externalJarWithDir)) {
                missingJars.add(extrernalJar);
            }
        }

        if (!missingJars.isEmpty()) {
            String title = "Es fehlen externe Jars.";
            String message = "Es fehlen die folgenden externe Jars:\n"
                    + CollectionsHelper.listListWithoutNumberAndBracket(missingJars);
            String beforeUrl = "Bitte besuchen sie die Seite";
            String url = SHORT_VOCBULARY_TRAINER_WEBPAGE_URL;
            String afterUrl = "laden Sie diese nach den dortigen Angaben herunter und "
                    + "kopieren Sie sie ins Verzeichnis\n\n"
                    + "    " + programDir
                    + "\n\n.";
            DialogWithCopyableUrl dialog = new DialogWithCopyableUrl(title, message,
                    beforeUrl, url, afterUrl, gui.getLocation(), gui.getProgramImage());
            dialog.setVisible(true);

            gui.appendMessage("    Abbruch wegen fehlender Jars.");
            gui.printSplashTextAndExit();
        }
    }

}
