package de.duehl.vocabulary.japanese.launcher.logic.install;

import de.duehl.basics.io.FileHelper;
import de.duehl.basics.system.OperationSystem;
import de.duehl.basics.system.SystemTools;
import de.duehl.basics.system.windows.WindowsShortcutCreator;
import de.duehl.basics.version.Version;
import de.duehl.html.download.binary.SimpleBinaryFileDownloader;
import de.duehl.swing.ui.GuiTools;
import de.duehl.vocabulary.japanese.common.chan.Chan;
import de.duehl.vocabulary.japanese.common.persistence.Options;
import de.duehl.vocabulary.japanese.common.persistence.SessionManager;
import de.duehl.vocabulary.japanese.common.tools.VocabularyPathHelper;
import de.duehl.vocabulary.japanese.common.website.VocaluaryTrainerWebsiteConstants;
import de.duehl.vocabulary.japanese.launcher.VocabularyTrainerLauncherVersion;
import de.duehl.vocabulary.japanese.launcher.ui.LauncherGui;

/**
 * Diese Klasse stellt den Installationsprozess für den Vokabeltrainers dar.
 *
 * Dieser wird im Launcher angestoßen, wenn es die Sessiondatei oder das Verzeichnis, in dem diese
 * erwartet wird, nicht gibt.
 *
 * @version 1.01     2024-12-08
 * @author Christian Dühl
 */

public class Installer {

    /** Die grafische Oberfläche des Launchers. */
    private final LauncherGui gui;

    /**
     * Das Verzeichnis in dem der Launcher und der Vokabeltrainer zusammen mit den externen Jars
     * liegen.
     */
    private final String programDir;

    /** Die Programmoptionen. */
    private Options options;

    /** Der Session-Manager. */
    private SessionManager sessionManager;

    /**
     * Konstruktor.
     *
     * @param gui
     *            Die grafische Oberfläche des Launchers.
     */
    public Installer(LauncherGui gui) {
        this.gui = gui;

        programDir = SystemTools.getCurrentWorkingDirectory();
    }

    /** Führt die Installation durch. */
    public void install() {
        createNewOptionsAndSessionManager();
        checkLauncherDirAkaProgrammPath();
        askUserAboutVocabularyDirectoryPath();
        writeOptions();
        installOperatingSystemDependingThings();
    }

    /** Erzeugt ein neues Optionenobjekt und den dieses verwaltenden Session-Manager. */
    private void createNewOptionsAndSessionManager() {
        gui.appendMessage("    erzeuge ein neues Optionen-Objekt ...");
        Version version000 = VocabularyPathHelper.createVersion000();
        options = new Options(version000);
        sessionManager = new SessionManager(version000, options);
    }

    /**
     * Prüft den Pfad, in dem der Launcher liegt und der später auch das Programmverzeichnis ist.
     */
    private void checkLauncherDirAkaProgrammPath() {
        gui.appendMessage("    überprüfe das Programmverzeichnis ...");

        checkDir(programDir, "Programmverzeichnis");
    }

    private void checkDir(String dir, String directoryDescription) {
        if (SystemTools.isPathInProtectedWindowsProgramDirs(dir)) {
            String title = "Verzeichnis liegt im geschützten Programmverzeichnis von Windows";
            String message = "Das " + directoryDescription + " liegt in einem der geschützten "
                    + "Programmverzeichnisse von Windows.\n\n"
                    + "Daher wird die Ausführung abgebrochen.\n"
                    + "Bitte nutzen sie ein anderes Verzeichnis wie z.B. unterhalb von "
                    + "C:\\meine_programme.";
            GuiTools.informUser(gui.getWindowAsComponent(), title, message);
            gui.appendMessage("    Abbruch wegen unzulässiger Pfade.");
            gui.printSplashTextAndExit();
        }
    }

    /** Fragt den Benutzer nach dem Pfad für die Vokabeln. */
    private void askUserAboutVocabularyDirectoryPath() {
        gui.appendMessage("    lege Vokabelverzeichnis fest ...");

        String vocabulariesPath = FileHelper.concatPathes(programDir, "vokabeln");

        String title = "Soll das Vokabelverzeichnis im Programmverzeichnis eingerichtet werden?";
        String message = "Soll das Vokabelverzeichnis im Programmverzeichnis, also unter\n\n"
                + "    "  + vocabulariesPath
                + "\n\n eingerichtet werden?";
        boolean usePathAsIs = GuiTools.askUser(gui.getWindowAsComponent(), title, message);
        if (!usePathAsIs) {
            vocabulariesPath = GuiTools.openDirectory(vocabulariesPath,
                    "Bitte wählen Sie das Vokabelverzeichnis aus", gui.getWindowAsComponent());
            if (vocabulariesPath.isBlank()) {
                gui.appendMessage("    Abbruch durch den Benutzer.");
                gui.printSplashTextAndExit();
            }
        }
        checkDir(vocabulariesPath, "Vokabelverzeichnis");

        FileHelper.createDirectoryWithMissingSubdirectoriesIfNotExists(vocabulariesPath);
        options.setVocabulariesPath(vocabulariesPath);
    }

    /** Schreibt die erzeugten Optionen raus. */
    private void writeOptions() {
        gui.appendMessage("    speichere Optionen ins Session-File ...");

        sessionManager.save();
    }

    private void installOperatingSystemDependingThings() {
        OperationSystem os = SystemTools.determineOperationSystem();
        if (os == OperationSystem.WINDOWS) {
            installWindowsThings();
        }
    }

    private void installWindowsThings() {
        downloadIconForWindows();
        createStartBatchWindows();
        createDesktopLinkIfUserWantsTo();
    }

    /** Der Name des Icons ohne Pfad. */
    private static final String ICON_BARE_FILENAME = "japanisch_deutsch.ico";

    private void downloadIconForWindows() {
        gui.appendMessage("    Lade das Icon herunter ...");

        String iconFileUrl = VocaluaryTrainerWebsiteConstants.BASE_URL + ICON_BARE_FILENAME;
        String localIconFilename = FileHelper.concatPathes(programDir, ICON_BARE_FILENAME);
        SimpleBinaryFileDownloader.downloadBinaryFile(iconFileUrl, localIconFilename);
    }

    /** Der Name der Batchdatei ohne Pfad. */
    private static final String BATCH_BARE_FILENAME = "run_" + Chan.getChanNameForFilenames()
            + "s_vokabel_trainer.bat";

    /**
     * Erzeugt eine Batchdatei, mit der der Launcher gestartet werden kann, auch wenn man einen
     * neueren Launcher herunterlädt. Der Launcher passt diese Batchdatei dann nämlich automatisch
     * an.
     */
    private void createStartBatchWindows() {
        gui.appendMessage("    Erzeuge Batchdatei zum Start des Launchers ...");

        String batchContent = """
                @echo off
                set dir=###PATH###
                set jar=%dir%de.duehl.vocabulary.japanese.launcher_###LAUNCHER_VERSION###.jar
                set classToStart=de.duehl.vocabulary.japanese.launcher.start.StartLauncher


                title Launcher für ###CHANNAME###s Vokabel-Trainer
                java -cp %jar% %classToStart%

                rem Wenn Java nur kopiert, aber nicht installiert wurde, statt dessen diese Zeilen
                rem anpassen und aktiv schalten:
                rem set java=c:\\Meine_Programme\\java\\jdk-17.0.1.12-hotspot\\bin\\java.exe
                rem %java% -cp %jar% %classToStart%

                rem pause
                """;

        if (programDir.endsWith("\\") || programDir.endsWith("/")) {
            batchContent = batchContent.replace("###PATH###", programDir);
        }
        else {
            batchContent = batchContent.replace("###PATH###", programDir + "\\");
        }

        batchContent = batchContent.replace("###CHANNAME###", Chan.CHAN_NAME);
        batchContent = batchContent.replace("###LAUNCHER_VERSION###",
                VocabularyTrainerLauncherVersion.getOnlyVersion());

        String batchFilename = FileHelper.concatPathes(programDir, BATCH_BARE_FILENAME);
        FileHelper.writeTextToFile(batchContent, batchFilename);
    }

    private void createDesktopLinkIfUserWantsTo() {
        String title = "Soll eine Verknüpfung auf dem Desktop erzeugt werden?";
        String message = "Soll eine Verknüpfung auf dem Desktop zum Starten des Vokabeltrainers "
                + "erzeugt werden?";
        boolean createLink = GuiTools.askUser(gui.getWindowAsComponent(), title, message);

        if (createLink) {
            gui.appendMessage("    Erzeuge Verknüpfung auf dem Desktop ...");
            createDesktopLink();
        }
        else {
            gui.appendMessage("    Erzeuge keine Verknüpfung auf dem Desktop.");
        }
    }

    private void createDesktopLink() {
        try {
            tryToCreateDesktopLink();
        }
        catch (Exception exception) {
            String title = "Fehler beim Versuch eine Verknüpfung auf dem Desktop zu erstellen";
            String message = exception.getMessage();
            GuiTools.informUser(gui.getWindowAsComponent(), title, message);
            gui.appendMessage("   " + title + ": " + message);
            gui.printSplashTextAndExit();
        }
    }

    private void tryToCreateDesktopLink() {
        String sourceFilename = FileHelper.concatPathes(programDir, BATCH_BARE_FILENAME);
        String linkName = Chan.CHAN_NAME + "s Vokabel Trainer.lnk";
        String iconFilename = FileHelper.concatPathes(programDir, ICON_BARE_FILENAME);

        WindowsShortcutCreator creator = new WindowsShortcutCreator(sourceFilename);
        creator.createDesktopShortcut(linkName);
        creator.setIcon(iconFilename);
        creator.setWorkingDirectory(programDir);
        creator.create();
    }

}
