package de.duehl.vocabulary.japanese.launcher.logic.selfcheck;

import de.duehl.basics.io.FileHelper;
import de.duehl.basics.system.SystemTools;
import de.duehl.basics.version.Version;
import de.duehl.html.download.binary.SimpleBinaryFileDownloader;
import de.duehl.swing.ui.GuiTools;
import de.duehl.vocabulary.japanese.common.website.VocaluaryTrainerWebsiteConstants;
import de.duehl.vocabulary.japanese.launcher.VocabularyTrainerLauncherVersion;

import static de.duehl.vocabulary.japanese.common.website.VocaluaryTrainerWebsiteConstants.NEWEST_LAUNCHER_VERSION_ON_SERVER_URL;
import static de.duehl.vocabulary.japanese.common.website.download.WebsiteFileContentDownloader.NO_DOWNLOAD_SUCCESS;
import static de.duehl.vocabulary.japanese.common.website.download.WebsiteFileContentDownloader.downloadFileViaUrlFromServerAndGetContentOfFirstLine;

/**
 * Diese Klasse prüft, ob der Launcher selbst aktualisiert werden soll.
 *
 * @version 1.01     2024-12-08
 * @author Christian Dühl
 */

public class LauncherSelfUpdateChecker {

    /** Der Anfang der Dateinamen der Jars mit dem Launcher. */
    public final static String LAUNCHER_JAR_NAME_START = "de.duehl.vocabulary.japanese.launcher_";


    /** Gibt an, ob der Launcher aktuell ist, und also danach fortfahren soll. */
    private boolean launcherIsActual;

    /** Die Version des Launchers für die Batchdateien. */
    private String launcherVersion;

    /** Führt die Prüfung durch. */
    public void check() {
        launcherIsActual = true;

        Version launcherVersionRunning = new VocabularyTrainerLauncherVersion().getVersion();

        String launcherVersionNumberOnServer = downloadLauncherVersionFromServer();
        if (launcherVersionNumberOnServer.equals(NO_DOWNLOAD_SUCCESS)) {
            // ignorieren wir
            launcherVersion = launcherVersionRunning.getVersion();
        }
        else {
            Version launcherVersionOnServer = new Version(launcherVersionNumberOnServer,
                    "interessiert nicht");
            if (launcherVersionOnServer.isNewerThan(launcherVersionRunning)) {
                downloadNewLauncherFromWebsite(launcherVersionNumberOnServer);
                informUserAboutNewerProgramVersion(launcherVersionRunning.getVersion(),
                        launcherVersionNumberOnServer);
                launcherVersion = launcherVersionOnServer.getVersion();
                launcherIsActual = false;
            }
            else {
                launcherVersion = launcherVersionRunning.getVersion();
            }
        }
    }

    /**
     * Lädt die erste Zeile der Datei mit der Version des Launchers auf dem Webserver herunter.
     *
     * Hatte das Herunterladen keinen Erfolg, so wird NO_DOWNLOAD_SUCCESS zurückgegeben.
     */
    private static String downloadLauncherVersionFromServer() {
        return downloadFileViaUrlFromServerAndGetContentOfFirstLine(
                NEWEST_LAUNCHER_VERSION_ON_SERVER_URL);
    }

    private void downloadNewLauncherFromWebsite(String versionNumberOnServer) {
        String jarBareFilename = createLocalBareJarFilenameFromVersion(versionNumberOnServer);
        String jarFileUrl = VocaluaryTrainerWebsiteConstants.BASE_URL + jarBareFilename;
        String dir = SystemTools.getCurrentWorkingDirectory();
        String localJarFilename = FileHelper.concatPathes(dir, jarBareFilename);
        SimpleBinaryFileDownloader.downloadBinaryFile(jarFileUrl, localJarFilename);
    }

    private static String createLocalBareJarFilenameFromVersion(String version) {
        return LAUNCHER_JAR_NAME_START + version + ".jar";
    }

    private void informUserAboutNewerProgramVersion(String verionNumberRunning,
            String versionNumberOnServer) {
        String title = "Es gibt eine neuere Version des Launchers";
        String message = "Es gibt eine neuere Version des Launchers!\n\n"
                + "Aktuelle Version des Launchers: " + verionNumberRunning + "\n"
                + "Neuere Version des Launchers: " + versionNumberOnServer+ "\n\n"
                + "Der neue Launcher wurde heruntergeladen, bitte starten Sie im Anschluss die "
                + "neue Version.";
        GuiTools.informUser(title, message);
    }

    /** Gibt an, ob der Launcher aktuell ist, und also danach fortfahren soll. */
    public boolean isLauncherActual() {
        return launcherIsActual;
    }

    /** Getter für die Version des Launchers für die Batchdateien. */
    public String getLauncherVersion() {
        return launcherVersion;
    }

}
