package de.duehl.vocabulary.japanese.launcher.ui.dialogs;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.KeyStroke;

import de.duehl.basics.text.html.HtmlTool;
import de.duehl.swing.ui.GuiTools;
import de.duehl.swing.ui.dialogs.PasswordDialog;
import de.duehl.swing.ui.dialogs.base.ModalDialogBase;
import de.duehl.swing.ui.key.BindKeysOnRootPane;
import de.duehl.swing.ui.layout.VerticalLayout;
import de.duehl.vocabulary.japanese.launcher.update.vocables.download.VocableUpdateUserPasword;

/**
 * Diese Klasse fragt den Benutzer, ob er eine neuere Version der Vokabeln von der Webseite
 * herunterladen und installieren möchte.
 *
 * @version 1.01     2024-12-07
 * @author Christian Dühl
 */

public class NewVocablesVersionOnlineDialog extends ModalDialogBase {

    private static final int DIALOG_WIDTH = 450;
    private static final int BIGGER_TEXT_FACTOR = 5;

    /** Die anzuzeigende Nachricht mit den Versionen. */
    private final String message;

    /** Gibt an, ob der Benutzer die neue Version herunterladen möchte. */
    private boolean userWantsToDownlad;

    /**
     * Konstruktor.
     *
     * @param localVocableVersion
     *            Die Version der lokal verwendeten Vokabeln.
     * @param websiteVocableVersion
     *            Die Version der Vokabeln auf dem Server.
     * @param parentLocation
     *            Die Position des Rahmens der Oberfläche, vor der dieser Dialog erzeugt wird.
     * @param programImage
     *            Das Icon für das Programm.
     */
    public NewVocablesVersionOnlineDialog(String localVocableVersion, String websiteVocableVersion,
            Point parentLocation, Image programImage) {
        super(parentLocation, programImage, "Es gibt eine neuere Version der Vokabeln");

        message = "Es gibt eine neuere Version der Vokabeln!\n\n"
                + "Aktuelle Version der Vokabeln: " + localVocableVersion + "\n"
                + "Neuere Version der Vokabeln: " + websiteVocableVersion;

        userWantsToDownlad = false;

        fillDialog();
    }

    /** Baut die Gui auf. */
    @Override
    protected void populateDialog() {
        add(createDummyPanel(), BorderLayout.NORTH);
        add(createCenterPart(), BorderLayout.CENTER);
        add(createButtonPart(), BorderLayout.SOUTH);

        keybindingsForPasswortDialog();
    }

    private Component createCenterPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new VerticalLayout(0, VerticalLayout.BOTH));

        panel.add(createMessagePart());
        panel.add(createOpenPasswordDialogLabel());

        return panel;
    }

    private Component createDummyPanel() {
        JPanel panel = new JPanel();
        panel.setPreferredSize(new Dimension(DIALOG_WIDTH, 0));
        return panel;
    }

    private Component createMessagePart() {
        String text = message + "\n\n";
        JLabel label = new JLabel(HtmlTool.htmlify(text));
        GuiTools.biggerFont(label, BIGGER_TEXT_FACTOR);
        return label;
    }

    private Component createOpenPasswordDialogLabel() {
        JLabel label = new JLabel("Einmalig das Passwort eingeben: Strg-Shift-P drücken.");
        label.setBorder(BorderFactory.createEmptyBorder(0,  5,  0,  5));
        return label;
    }

    private Component createButtonPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        panel.add(createQuitButton(), BorderLayout.WEST);
        panel.add(createDownloadButton(), BorderLayout.EAST);

        return panel;
    }

    private Component createQuitButton() {
        JButton button = new JButton("Abbrechen");
        button.addActionListener(e -> closeDialog());
        return button;
    }

    private Component createDownloadButton() {
        JButton button = new JButton("Download starten");
        GuiTools.biggerFont(button, BIGGER_TEXT_FACTOR);
        button.addActionListener(e -> download());
        return button;
    }

    private void download() {
        userWantsToDownlad = true;
        closeDialog();
    }

    private void keybindingsForPasswortDialog() {
        KeyStroke keyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_P,
                InputEvent.CTRL_DOWN_MASK | InputEvent.SHIFT_DOWN_MASK);
        String actionMapKey = "Strg-Shift-P";
        Action action = BindKeysOnRootPane.runnableToAction(() -> showPasswordDialog());

        JRootPane rootPane = getRootPane();
        InputMap inputMap = rootPane.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
        inputMap.put(keyStroke, actionMapKey);
        rootPane.getActionMap().put(actionMapKey, action);
    }

    private void showPasswordDialog() {
        String title = "Bitte das Passwort eingeben";
        String message = "Bitte das Passwort für die Vokabeln mit Aussprache eingeben.";
        PasswordDialog dialog = new PasswordDialog(title, message);
        dialog.setVisible(true);
        String password = dialog.getPassword();
        if (!password.isBlank()) {
            VocableUpdateUserPasword.savePassword(password);
        }
    }

    /** Gibt an, ob der Benutzer die neue Version herunterladen möchte. */
    public boolean userWantsToDownload() {
        return userWantsToDownlad;
    }

}
