package de.duehl.vocabulary.japanese.launcher.update.vocables;

import javax.swing.SwingUtilities;

import de.duehl.basics.text.Text;
import de.duehl.swing.ui.GuiTools;
import de.duehl.swing.ui.dialogs.base.AbstractDialogBase;
import de.duehl.vocabulary.japanese.common.persistence.Options;
import de.duehl.vocabulary.japanese.common.website.update.UpdateTools;
import de.duehl.vocabulary.japanese.common.website.update.vocables.VocablesActualityChecker;
import de.duehl.vocabulary.japanese.launcher.ui.dialogs.NewVocablesVersionOnlineDialog;
import de.duehl.vocabulary.japanese.launcher.update.vocables.download.VocablesDownloader;
import de.duehl.vocabulary.japanese.launcher.update.vocables.error.FatalErrorHandler;

/**
 * Diese Klasse kümmert sich um Updates der Vokabularien aus dem Launcher heraus.
 *
 * @version 1.01     2024-12-11
 * @author Christian Dühl
 */

public class VocablesUpdater {

    private static final boolean DEBUG = false;


    /** Die Programmoptionen. */
    private final Options options;

    /** Die grafische Oberfläche des Vokabel-Trainers bzw. der Gui des Launchers. */
    private final AbstractDialogBase gui;

    /** Gibt an, ob der Launcher das erste Mal auf diesem Computer gestartet wurde. */
    private boolean firstLaunchOnThisComputer;

    /** Der Pfad zu den Vokabularien und den Sound-Files. */
    private final String vocabulariesPath;

    /** Wird nach der Prüfung und dem ggf. folgenden Download ausgeführt. */
    private Runnable runAfterCheckAndDownload;

    /** Behandelt fatale Fehler. */
    private FatalErrorHandler fatalErrorHandler;

    /**
     * Konstruktor.
     *
     * @param options
     *            Die Programmoptionen.
     * @param gui
     *            Die grafische Oberfläche des Vokabel-Trainers bzw. der Gui des Launchers.
     * @param firstLaunchOnThisComputer
     *            Gibt an, ob der Launcher das erste Mal auf diesem Computer gestartet wurde.
     */
    public VocablesUpdater(Options options, AbstractDialogBase gui,
            boolean firstLaunchOnThisComputer, Runnable runAfterCheckAndDownload) {
        this.options = options;
        this.gui = gui;
        this.firstLaunchOnThisComputer = firstLaunchOnThisComputer;
        this.runAfterCheckAndDownload = runAfterCheckAndDownload;

        vocabulariesPath = options.getVocabulariesPath();
    }

    /** Prüft auf eine aktuellere Version der Vokabularien. */
    public void update() {
        say("update()");
        VocablesActualityChecker checker = new VocablesActualityChecker(
                options.getVocabulariesPath());
        checker.check();
        if (checker.isVersionOnWebserverDetermined()) {
            say("    Die Version auf dem Webserver wurde ermittelt.");
            String localVocableVersion = checker.getLocalVocableVersion();
            String websiteVocableVersion = checker.getWebsiteVocableVersion();
            say("        localVocableVersion = " + localVocableVersion);
            say("        websiteVocableVersion = " + websiteVocableVersion);
            if (checker.isActual()) {
                say("        Ist aktuell.");
                if (firstLaunchOnThisComputer) {
                    // Kann sein, wenn der Benutzer sie bereits von Hand installiert hat.
                    informUserAboutVocableVersionIsActual(localVocableVersion);
                }
                runAfterCheckAndDownload();
            }
            else {
                say("        Ist NICHT aktuell.");
                boolean download = askUserIfHeWantsToDownload(localVocableVersion,
                        websiteVocableVersion);
                say("        download = " + download);
                if (download) {
                    downloadNewerVersion(websiteVocableVersion);
                }
                else {
                    runAfterCheckAndDownload();
                }
            }
        }
        else if (firstLaunchOnThisComputer) {
            UpdateTools.informUserAboutDownloadFailure("der Vokabeln", gui.getLocation(),
                    gui.getProgramImage());
        }
    }

    private boolean askUserIfHeWantsToDownload(String localVocableVersion,
            String websiteVocableVersion) {
        NewVocablesVersionOnlineDialog dialog = new NewVocablesVersionOnlineDialog(
                localVocableVersion, websiteVocableVersion, gui.getLocation(),
                gui.getProgramImage());
        dialog.setVisible(true);
        return dialog.userWantsToDownload();
    }

    private void downloadNewerVersion(String websiteVocableVersion) {
        say("downloadNewerVersion()");
        gui.startLongTimeProcess(
                "Lade aktuellere Version der Vokabeln herunter und installiere sie");
        fatalErrorHandler = new FatalErrorHandler(gui);
        new Thread(() -> downloadNewerVersionInOwnThread(websiteVocableVersion)).start();
    }

    private void downloadNewerVersionInOwnThread(String websiteVocableVersion) {
        say("downloadNewerVersionInOwnThread()");
        try {
            tryToDownloadNewerVersion(websiteVocableVersion);
        }
        catch (Exception exception) {
            showError(exception);
        }
    }

    private void tryToDownloadNewerVersion(String websiteVocableVersion) {
        say("tryToDownloadNewerVersion()");

        VocablesDownloader downloader = new VocablesDownloader(vocabulariesPath,
                websiteVocableVersion, runAfterCheckAndDownload, gui,
                fatalErrorHandler);
        downloader.download();

        SwingUtilities.invokeLater(() -> gui.endLongTimeProcess());
    }


    private void runAfterCheckAndDownload() {
        say("runAfterCheckAndDownload()");
        runAfterCheckAndDownload.run();
    }

    private void informUserAboutVocableVersionIsActual(String vocableVersion) {
        String title = "Die Vokabeln ist aktuell";
        String message = "Die verwendete Version der Vokabeln " + vocableVersion
                + " ist aktuell.";
        GuiTools.informUserInEdt(gui.getWindowAsComponent(), title, message);
    }

    private void showError(Exception exception) {
        fatalErrorHandler.showError(exception);
    }

    public static void say(String message) {
        if (DEBUG) {
            Text.say("DEBUG: " + message);
        }
    }

}
