package de.duehl.vocabulary.japanese.launcher.update.vocables.download;

import java.util.ArrayList;
import java.util.List;

import de.duehl.basics.io.FileHelper;
import de.duehl.basics.text.handling.BooleanHandling;
import de.duehl.html.download.binary.SimpleBinaryFileDownloader;
import de.duehl.swing.ui.GuiTools;
import de.duehl.swing.ui.dialogs.base.AbstractDialogBase;
import de.duehl.vocabulary.japanese.common.website.VocaluaryTrainerWebsiteConstants;
import de.duehl.vocabulary.japanese.launcher.update.vocables.error.FatalErrorHandler;
import de.duehl.zipwithzip4j.unzip.UnzipWithZip4J2;

import static de.duehl.vocabulary.japanese.common.website.update.vocables.VocabularyZipFilenameCreator.createZipBareFilenameWithMp3;
import static de.duehl.vocabulary.japanese.common.website.update.vocables.VocabularyZipFilenameCreator.createZipBareFilenameWithoutMp3;
import static de.duehl.vocabulary.japanese.launcher.update.vocables.VocablesUpdater.say;

/**
 * Diese Klasse lädt im Rahmen des Updates der Vokabularien die neue Version herunter.
 *
 * Wird nur vom Launcher aus verwendet.
 *
 * @version 1.01     2024-12-11
 * @author Christian Dühl
 */

public class VocablesDownloader {

    private static final boolean DELETE_FILES = true;

    /** Der Pfad zu den Vokabularien und den Sound-Files. */
    private final String vocabulariesPath;

    /** Die Version der Vokabeln auf der Webseite. */
    private final String websiteVocableVersion;

    /**
     * Wird nach der Prüfung und dem ggf. folgenden Download ausgeführt, wenn die Klasse aus dem
     * Launcher aufgerufen wird.
     */
    private final Runnable runAfterCheckAndDownload;

    /** Die grafische Oberfläche des Vokabel-Trainers bzw. der Gui des Launchers. */
    private final AbstractDialogBase gui;

    /** Behandelt fatale Fehler. */
    private FatalErrorHandler fatalErrorHandler;

    /** Liste mit den lokalen Dateien, die am Ende wieder gelöscht werden sollen. */
    private final List<String> filesToRemoveLater;

    /**
     * Konstruktor.
     *
     * @param vocabulariesPath
     *            Der Pfad zu den Vokabularien und den Sound-Files.
     * @param websiteVocableVersion
     *            Die Version der Vokabeln auf der Webseite.
     * @param calledByLauncher
     *            Gibt an, ob die Klasse aus dem Launcher aufgerufen wird.
     * @param runAfterCheckAndDownload
     *            Wird nach der Prüfung und dem ggf. folgenden Download ausgeführt, wenn die Klasse
     *            aus dem Launcher aufgerufen wird.
     * @param gui
     *            Die grafische Oberfläche des Vokabel-Trainers bzw. der Gui des Launchers.
     * @param fatalErrorHandler
     *            Behandelt fatale Fehler.
     */
    public VocablesDownloader(String vocabulariesPath, String websiteVocableVersion,
            Runnable runAfterCheckAndDownload, AbstractDialogBase gui,
            FatalErrorHandler fatalErrorHandler) {
        this.vocabulariesPath = vocabulariesPath;
        this.websiteVocableVersion = websiteVocableVersion;
        this.runAfterCheckAndDownload = runAfterCheckAndDownload;
        this.gui = gui;
        this.fatalErrorHandler = fatalErrorHandler;

        filesToRemoveLater = new ArrayList<>();
    }

    /** Führt den Download durch. */
    public void download() {
        say("VocablesDownloader: download()");
        prepare();
        String password = VocableUpdateUserPasword.readPassword();
        say("    passwort : '" + password + "'");
        if (password.isEmpty()) {
            downloadVersionWithoutMP3();
        }
        else {
            downloadVersionWithMP3(password);
        }
    }

    private void prepare() {
        say("prepare()");
        cleanUpVocableDir();
    }

    private void cleanUpVocableDir() {
        say("cleanUpVocableDir()");
        FileHelper.deleteTree(vocabulariesPath);
        FileHelper.createDirectoryIfNotExists(vocabulariesPath);
    }

    private void downloadVersionWithoutMP3() {
        say("downloadVersionWithoutMP3()");
        String zipBareFilename = createZipBareFilenameWithoutMp3(websiteVocableVersion);
        String localZipFilename = downloadZipfile(zipBareFilename);
        filesToRemoveLater.add(localZipFilename);
        unzipWithoutPassword(localZipFilename);
        afterUnzip();
    }

    /**
     * Lädt die gewünschte Zip-Datei vom Webserver herunter.
     *
     * @param zipBareFilename
     *            Der Name der Zip-Datei ohne Pfad.
     * @return Der Name der heruntergeladenen Zip-Datei mit PFad.
     */
    private String downloadZipfile(String zipBareFilename) {
        say("downloadZipfile()");
        String zipFileUrl = VocaluaryTrainerWebsiteConstants.BASE_URL + zipBareFilename;
        String localZipFilename = FileHelper.concatPathes(vocabulariesPath, zipBareFilename);
        SimpleBinaryFileDownloader.downloadBinaryFile(zipFileUrl, localZipFilename);
        return localZipFilename;
    }

    private void unzipWithoutPassword(String localZipFilename) {
        say("unzipWithoutPassword()");
        UnzipWithZip4J2 unzip = new UnzipWithZip4J2(localZipFilename, vocabulariesPath);
        unzip.unzip();
    }

    private void downloadVersionWithMP3(String password) {
        say("downloadVersionWithMP3()");
        try {
            tryToDownloadVersionWithMP3(password);
        }
        catch (Exception exception) {
            handleDownloadVersionWithMP3Exception(exception);
        }
    }

    private void tryToDownloadVersionWithMP3(String password) {
        say("tryToDownloadVersionWithMP3()");
        String zipBareFilename = createZipBareFilenameWithMp3(websiteVocableVersion);
        String localZipFilename = downloadZipfile(zipBareFilename);
        say("    localZipFilename = " + localZipFilename);
        filesToRemoveLater.add(localZipFilename);
        unzipWithPassword(localZipFilename, password);
        afterUnzip();
    }

    /**
     * Entpackt die Dateien mit MP3 als Zip im Zip, weil zip4j aus lizenzrechtlichen Gründen die
     * Dateiliste nicht sichern kann, anders als 7zip.
     *
     * @param localZipFilename
     *            Der Name des lokal abgelegten Archivs.
     * @param password
     *            Das Passwort für die Verschlüsselung.
     */
    private void unzipWithPassword(String localZipFilename, String password) {
        say("unzipWithPassword()");

        String dirToExtractTheInnerZipFile = FileHelper.getDirName(localZipFilename);
        UnzipWithZip4J2 outerUnzip = new UnzipWithZip4J2(localZipFilename,
                dirToExtractTheInnerZipFile);
        outerUnzip.cypher(password);
        outerUnzip.unzip();
        say("    Äußeres Zipfile erfolgreich ausgepackt.");

        String innerZipFilename = FileHelper.insertBeforeExtension(localZipFilename, "_inner");
        filesToRemoveLater.add(innerZipFilename);
        UnzipWithZip4J2 innerUnzip = new UnzipWithZip4J2(innerZipFilename, vocabulariesPath);
        innerUnzip.cypher(password);
        innerUnzip.unzip();
        say("    Inneres Zipfile erfolgreich ausgepackt.");
    }

    private void handleDownloadVersionWithMP3Exception(Exception exception) {
        say("handleDownloadVersionWithMP3Exception() - message: " + exception.getMessage());
        if (exception.getMessage().contains("Wrong Password")) {
            say("    wegen falschem Passwort");
            handleWrongPassword();
        }
        else {
            say("    aus anderen Gründen");
            throw new RuntimeException(exception);
            // wird weiter oben aufgefangen und als Fehlerdialog angezeigt
        }
    }

    private void handleWrongPassword() {
        say("handleWrongPassword()");
        String title = "Falsches Passwort";
        String message = "Das Passwort war falsch.\n\n"
                + "Sollen die Vokabeln ohne MP3 heruntergeladen werden?\n\n"
                + "Anderenfalls wird das Programm beendet.";
        boolean downloadWithoutMp3 = GuiTools.askUserDefaultNo(gui.getWindowAsComponent(), title,
                message);
        say("    downloadWithoutMp3 : " + BooleanHandling.booleanToString(downloadWithoutMp3));
        if (downloadWithoutMp3) {
            downloadVersionWithoutMP3();
        }
        else {
            fatalErrorHandler.endLongTimeProcessAndCloseGui();
        }
    }

    private void afterUnzip() {
        say("afterUnzip()");
        showProgramHasToBeRestartetInfo();
        cleanUp();
        runAfterCheckAndDownload();
    }

    private void showProgramHasToBeRestartetInfo() {
        say("showProgramHasToBeRestartetInfo()");
        String title = "Zur Aktualisierung neu Starten";
        String message = "Damit die aktualisierten Vokabeln eingelesen werden,\n"
                + "bitte das Programm neu starten.";
        GuiTools.informUserInEdt(gui.getWindowAsComponent(), title, message);
    }

    private void cleanUp() {
        say("cleanUp()");
        if (DELETE_FILES) {
            for (String filename : filesToRemoveLater) {
                FileHelper.deleteFileIfExistent(filename);
            }
        }
    }

    private void runAfterCheckAndDownload() {
        say("runAfterCheckAndDownload()");
        runAfterCheckAndDownload.run();
    }

}
